@@ -59,12 +59,13 @@ main = do
 
   conf  <- doOpts c o
   fs    <- initFont d (font conf)
+  fl    <- mapM (initFont d) (additionalFonts conf)
   cls   <- mapM (parseTemplate conf) (splitTemplate conf)
   sig   <- setupSignalHandler
   vars  <- mapM (mapM $ startCommand sig) cls
   (r,w) <- createWin d fs conf
   let ic = Map.empty
-  startLoop (XConf d r w fs ic conf) sig vars
+  startLoop (XConf d r w (fs:fl) ic conf) sig vars
 
 -- | Splits the template in its parts
 splitTemplate :: Config -> [String]
@@ -121,6 +122,7 @@ data Opts = Help
           | Font       String
           | BgColor    String
           | FgColor    String
+          | Alpha      String
           | T
           | B
           | D
@@ -144,6 +146,8 @@ options =
       "The foreground color. Default grey"
     , Option "i" ["iconroot"] (ReqArg IconRoot "path")
       "Root directory for icon pattern paths. Default '.'"
+    , Option "A" ["alpha"] (ReqArg Alpha "alpha")
+      "The transparency: 0 is transparent, 255 is opaque. Default: 255"
     , Option "o" ["top"] (NoArg T) "Place xmobar at the top of the screen"
     , Option "b" ["bottom"] (NoArg B)
       "Place xmobar at the bottom of the screen"
@@ -200,6 +204,7 @@ doOpts conf (o:oo) =
     Font s -> doOpts' (conf {font = s})
     BgColor s -> doOpts' (conf {bgColor = s})
     FgColor s -> doOpts' (conf {fgColor = s})
+    Alpha n -> doOpts' (conf {alpha = read n})
     T -> doOpts' (conf {position = Top})
     B -> doOpts' (conf {position = Bottom})
     D -> doOpts' (conf {overrideRedirect = False})
