@@ -14,6 +14,7 @@
 
 module Plugins.Monitors.Thermal where
 
+import Control.Monad (liftM)
 import qualified Data.ByteString.Lazy.Char8 as B
 import Plugins.Monitors.Common
 import System.Posix.Files (fileExist)
@@ -32,11 +33,9 @@ runThermal args = do
     let zone = head args
         file = "/proc/acpi/thermal_zone/" ++ zone ++ "/temperature"
     exists <- io $ fileExist file
-    case exists of
-         False  -> return $ "Thermal (" ++ zone ++ "): N/A"
-         True   -> do number <- io $ B.readFile file
-                                     >>= return . (read :: String -> Int)
-                                                . stringParser (1, 0)
-                      thermal <- showWithColors show number
-                      parseTemplate [  thermal ]
+    if exists
+        then do number <- io $ liftM ((read :: String -> Int) . stringParser (1, 0)) (B.readFile file)
+                thermal <- showWithColors show number
+                parseTemplate [  thermal ]
+        else return $ "Thermal (" ++ zone ++ "): N/A"
 
