@@ -1,7 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Plugins.Monitors.Volume
--- Copyright   :  (c) 2011, 2013 Thomas Tuegel
+-- Copyright   :  (c) 2011, 2013, 2015 Thomas Tuegel
 -- License     :  BSD-style (see LICENSE)
 --
 -- Maintainer  :  Jose A. Ortega Ruiz <jao@gnu.org>
@@ -122,13 +122,7 @@ formatDb opts dbi = do
 runVolume :: String -> String -> [String] -> Monitor String
 runVolume mixerName controlName argv = do
     opts <- io $ parseOpts argv
-    (lo, hi, val, db, sw) <- io $ withMixer mixerName $ \mixer -> do
-        control <- getControlByName mixer controlName
-        (lo, hi) <- liftMaybe $ getRange <$> volumeControl control
-        val <- getVal $ volumeControl control
-        db <- getDB $ volumeControl control
-        sw <- getSw $ switchControl control
-        return (lo, hi, val, db, sw)
+    (lo, hi, val, db, sw) <- io readMixer
     p <- liftMonitor $ liftM3 formatVol lo hi val
     b <- liftMonitor $ liftM3 formatVolBar lo hi val
     v <- liftMonitor $ liftM3 formatVolVBar lo hi val
@@ -139,6 +133,16 @@ runVolume mixerName controlName argv = do
 
   where
 
+    readMixer =
+      AE.catch (withMixer mixerName $ \mixer -> do
+                   control <- getControlByName mixer controlName
+                   (lo, hi) <- liftMaybe $ getRange <$> volumeControl control
+                   val <- getVal $ volumeControl control
+                   db <- getDB $ volumeControl control
+                   sw <- getSw $ switchControl control
+                   return (lo, hi, val, db, sw))
+                (const $ return (Nothing, Nothing, Nothing, Nothing, Nothing))
+
     volumeControl :: Maybe Control -> Maybe Volume
     volumeControl c = join $
            (playback . volume <$> c) `mplus` (common . volume <$> c)
@@ -154,18 +158,19 @@ runVolume mixerName controlName argv = do
     liftMonitor Nothing = unavailable
     liftMonitor (Just m) = m
 
+    channel v r = AE.catch (getChannel FrontLeft v) (const $ return $ Just r)
+
     getDB :: Maybe Volume -> IO (Maybe Integer)
     getDB Nothing = return Nothing
-    getDB (Just v) = AE.catch (getChannel FrontLeft $ dB v)
-                              (const $ return $ Just 0)
+    getDB (Just v) = channel (dB v) 0
 
     getVal :: Maybe Volume -> IO (Maybe Integer)
     getVal Nothing = return Nothing
-    getVal (Just v) = getChannel FrontLeft $ value v
+    getVal (Just v) = channel (value v) 0
 
     getSw :: Maybe Switch -> IO (Maybe Bool)
     getSw Nothing = return Nothing
-    getSw (Just s) = getChannel FrontLeft s
+    getSw (Just s) = channel s False
 
     getFormatDB :: VolumeOpts -> Maybe Integer -> Monitor String
     getFormatDB _ Nothing = unavailable
